unit uMain;
{
  * Copyright 2015 E Spelt for test project stuff
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.

  * Implemented by E. Spelt for Delphi
  *
  * Modified by Jeehoon.Imp.Park@gmail.com
}
interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs, FMX.Objects,
  FMX.StdCtrls, FMX.Media, FMX.Platform, FMX.MultiView, FMX.ListView.Types,
  FMX.ListView, FMX.Layouts, System.Actions, FMX.ActnList, FMX.TabControl,
  FMX.ListBox, FMX.ScrollBox, FMX.Memo, Threading,
  FMX.Controls.Presentation,
  BarcodeFormat, ReadResult, ScanManager;

type
  TMainForm = class(TForm)
    btnCamera: TButton;
    imgCamera: TImage;
    Layout2: TLayout;
    ToolBar3: TToolBar;
    CameraComponent1: TCameraComponent;
    lbBarcodes: TListBox;
    Label1: TLabel;
    btnLoadText: TButton;
    OpenDialog1: TOpenDialog;
    procedure btnCameraClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);

    procedure FormDestroy(Sender: TObject);
    procedure CameraComponent1SampleBufferReady(Sender: TObject; const ATime: TMediaTime);
    procedure btnLoadTextClick(Sender: TObject);
  private
    FScanManager: TScanManager;
    FScanInProgress: Boolean;
    procedure GetImage;
    function AppEvent(AAppEvent: TApplicationEvent; AContext: TObject): Boolean;
    procedure CameraActive(AActive: boolean);
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$IFDEF ANDROID}
uses
  Androidapi.Helpers, FMX.Helpers.Android, Androidapi.JNI.Widget,
  Androidapi.JNI.Hardware, FMX.Media.Android, System.TypInfo;
{$ENDIF}

{$R *.fmx}

procedure ShowToast(const sMsg: string);
begin
{$IFDEF ANDROID}
  CallInUiThread(
    procedure
    begin
      TJToast.JavaClass.makeText(SharedActivityContext, StrToJCharSequence(sMsg), TJToast.JavaClass.LENGTH_SHORT).show;
    end);
{$ELSE}
  ShowMessage(sMsg);
{$ENDIF}
end;

function TMainForm.AppEvent(AAppEvent: TApplicationEvent; AContext: TObject): Boolean;
begin
  case AAppEvent of
    //TApplicationEvent.WillBecomeInactive,
    TApplicationEvent.EnteredBackground,
    TApplicationEvent.WillTerminate:
      CameraActive(False);
  end;
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
  AppEventSvc: IFMXApplicationEventService;
begin
  { Add platform service to see camera state. }
  if TPlatformServices.Current.SupportsPlatformService(IFMXApplicationEventService, IInterface(AppEventSvc)) then
    AppEventSvc.SetApplicationEventHandler(AppEvent);

  FScanManager := TScanManager.Create(TBarcodeFormat.Auto, nil);

{$IFDEF CPUARM}
  btnLoadText.Visible := false;
{$ENDIF}
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  FScanManager.Free;
end;

procedure TMainForm.CameraActive(AActive: boolean);
begin
  CameraComponent1.Active := AActive;
  if CameraComponent1.Active then
    btnCamera.Text := 'Stop'
  else
    btnCamera.Text := 'Start';
end;

procedure TMainForm.btnCameraClick(Sender: TObject);
var
  Setting: TVideoCaptureSetting;
begin
  if CameraComponent1.Active then
    CameraActive(False)
  else
  begin
    CameraActive(False);

    CameraComponent1.Kind := FMX.Media.TCameraKind.BackCamera;
    CameraComponent1.FocusMode := TFocusMode.ContinuousAutoFocus;
    CameraComponent1.CaptureSettingPriority := TVideoCaptureSettingPriority.Resolution;
    CameraComponent1.Quality := TVideoCaptureQuality.HighQuality;
    CameraComponent1.Quality := TVideoCaptureQuality.MediumQuality;

    Setting := CameraComponent1.CaptureSetting;
    Label1.Text := Format('%dx%d/%.0ffps', [Setting.Width, Setting.Height, Setting.FrameRate]);

    lbBarcodes.Items.Clear;

    CameraActive(True);
  end;
end;

procedure TMainForm.btnLoadTextClick(Sender: TObject);
var
  ReadResult1: TReadResult;
begin
  if not OpenDialog1.Execute then exit;
  imgCamera.Bitmap.LoadFromFile(OpenDialog1.FileName);

  TTask.Run(
    procedure
    begin
      try
        try
          FScanInProgress := True;
          ReadResult1 := FScanManager.Scan(imgCamera.Bitmap);
          FScanInProgress := False;
        except on E: Exception do
          begin
            TThread.Synchronize(nil,
              procedure
              begin
                ShowToast(E.Message);
              end);
            exit;
          end;
        end;

        if ReadResult1 <> nil then
        begin
          TThread.Synchronize(nil,
            procedure
            var
              iPos: integer;
            begin
              lbBarcodes.Items.Insert(0, ReadResult1.Text);
              ShowToast('SCANNED: ' + ReadResult1.Text);
            end);
          FreeAndNil(ReadResult1);
        end;
      finally
        FScanInProgress := False;
      end;
    end);
end;

procedure TMainForm.CameraComponent1SampleBufferReady(Sender: TObject; const ATime: TMediaTime);
begin
  TThread.Synchronize(TThread.CurrentThread, GetImage);
end;

procedure TMainForm.GetImage;
var
  scanBitmap, tempBitmap: TBitmap;
  ReadResult1: TReadResult;
  iNewHeight: Integer;
  ARect: TRect;
begin
  tempBitmap := TBitmap.Create;
  try
    CameraComponent1.SampleBufferToBitmap(tempBitmap, True);
    iNewHeight := Trunc(tempBitmap.Width * imgCamera.Size.Height / imgCamera.Size.Width);
    ARect := Rect(0, (tempBitmap.Height-iNewHeight) div 2, tempBitmap.Width, tempBitmap.Height - (tempBitmap.Height-iNewHeight) div 2);
    imgCamera.Bitmap.SetSize(ARect.Width, ARect.Height);
    imgCamera.Bitmap.CopyFromBitmap(tempBitmap, ARect, 0, 0);
  finally
    tempBitmap.Free;
  end;

  if (FScanInProgress) then exit;

  scanBitmap := TBitmap.Create;
  scanBitmap.Assign(imgCamera.Bitmap);

  TTask.Run(
    procedure
    begin
      try
        try
          FScanInProgress := True;
          ReadResult1 := FScanManager.Scan(scanBitmap);
          FScanInProgress := False;
        except on E: Exception do
          begin
            TThread.Synchronize(nil,
              procedure
              begin
                ShowToast(E.Message);
              end);
            exit;
          end;
        end;

        if ReadResult1 <> nil then
        begin
          TThread.Synchronize(nil,
            procedure
            var
              iPos: integer;
            begin
              iPos := lbBarcodes.Items.IndexOf(ReadResult1.Text);
              if iPos < 0 then
              begin
                lbBarcodes.Items.Insert(0, ReadResult1.Text);
                ShowToast('SCANNED: ' + ReadResult1.Text);
              end
              else
                lbBarcodes.ItemIndex := iPos;
            end);
          FreeAndNil(ReadResult1);
        end;
      finally
        scanBitmap.Free;
        FScanInProgress := False;
      end;
    end);
end;

end.
